require "import"
import "android.widget.*"
import "android.view.*"
import "cjson"

messages = {}

function urlEncode(text)
  text = tostring(text)
  text = text:gsub("\n", "\r\n")
  text = text:gsub("([^%w %-%_%.%~])", function(c)
    return string.format("%%%02X", string.byte(c))
  end)
  text = text:gsub(" ", "+")
  return text
end

function cleanText(text)
  local success, result = pcall(cjson.decode, text)
  if success and result then
    return result.answer or "Text is not available."
   else
    return "Text is not available."
  end
end

function addMessage(container, title, message)
  local fullMessage = title .. " " .. message
  local textView = TextView(service)
  container.addView(textView)
  textView.setText(fullMessage)
  table.insert(messages, fullMessage)
  textView.setTextColor(0xFFFFFFFF)
  textView.setTextSize(16)

  textView.setOnLongClickListener(View.OnLongClickListener{
    onLongClick = function(view)
      view.performHapticFeedback(HapticFeedbackConstants.LONG_PRESS)
      local text = tostring(textView.getText())
      local codes = {}
      for full_pattern, pattern_content in string.gmatch(text, "(.-)\n(.-)\n") do
        table.insert(codes, pattern_content)
      end
      if #codes > 0 then
        for i, code in ipairs(codes) do
          print(code)
        end
       else
        service.copy(text)
      end
      return true
    end
  })

  if message == "ChatGPT is typing..." then
    container.removeViewAt(container.getChildCount() - 1)
  end
end

layout = {
  LinearLayout;
  orientation = "vertical";
  layout_width="match_parent";
  layout_height="match_parent";
  backgroundColor = "#000000";
  {
    RelativeLayout;
    layout_width="match_parent";
    layout_height="wrap_content";
    backgroundColor = "#000000";
    {
      EditText;
      hint = "Pesan";
      id = "question";
      InputType = "textMultiLine";
      layout_width="match_parent";
      layout_height="wrap_content";
      layout_alignParentLeft=true;
      layout_marginRight="100dp";
    };
    {
      Button;
      text = ">";
      contentDescription = "Kirim";
      id = "KirimTombol";
      textColor = "#000000";
      backgroundColor = "#ffffff";
      layout_width="40sp";
      layout_height="wrap_content";
      layout_alignParentRight=true;
      onClick = function(view)
        local questionText = question.getText().toString()
        if questionText == "" then
          service.postSpeak(200, "Silakan Masukkan pesan Anda terlebih dahulu")
          return
        end
        local encodedQuestion = urlEncode(questionText)
        local url = "https://chatgpt.apinepdev.workers.dev/?question=" .. encodedQuestion
        addMessage(messagesContainer, "You:", questionText)
        service.postSpeak(200, "Chat GPT Sedang mengetik...")
        Http.get(url, function(status, data)
          if status == 200 then
            local cleanedData = cleanText(data)
            addMessage(messagesContainer, "ChatGPT:", cleanedData)
            service.postSpeak(200, cleanedData)
            question.setText("")
           else
            local errorMessage = "An error occurred while processing the request."
            addMessage(messagesContainer, "Error:", errorMessage)
            service.postSpeak(200, errorMessage)
          end
        end)
      end;
    };
  };
  {
    Button;
    text = "+";
    backgroundColor = "#000000";
    contentDescription = "Chat baru";
    layout_width = "match_parent";
    layout_height = "wrap_content";
    onClick = function()
      messagesContainer.removeAllViews()
      table.clear(messages)
    end;
  };
  {
    Button;
    text = "[■]";
    backgroundColor = "#000000";
    contentDescription = "Simpan menjadi file";
    layout_width = "match_parent";
    layout_height = "wrap_content";
    onClick = function()
      local save_layout = {
        LinearLayout;
        orientation = "vertical";
        layout_width="match_parent";
        layout_height="match_parent";
        backgroundColor = "#000000";
        {
          EditText;
          hint = "Tulis nama file";
          id = "fileEdit";
          layout_width="match_parent";
          layout_height="wrap_content";
          layout_marginLeft = "11dp";
          layout_marginRight = "16dp";
        };
        {
          Button;
          text = "Simpan";
          backgroundColor = "#000000";
          layout_width = "match_parent";
          layout_height = "wrap_content";
          onClick = function()
            local name = fileEdit.getText().toString()
            save_dlg.dismiss()
            if name ~= "" then
              local filename = "/sdcard/解说/SavedChats/" .. name .. ".txt"
              os.execute("mkdir -p /sdcard/解说/SavedChats")
              local file = io.open(filename, "w")
              if #messages > 0 then
                for i, message in ipairs(messages) do
                  file:write(message .. "\n")
                end
                file:close()
                service.postSpeak(200, "File berhasil disimpan")
               else
                service.postSpeak(200, "Anda tidak bisa menyimpan file kosong")
              end
             else
              service.postSpeak(200, "Tuliskan nama file terlebih dahulu")
            end
          end;
        };
      }
      save_dlg = LuaDialog()
      save_dlg.setTitle("Simpan obrolan")
      save_dlg.setView(loadlayout(save_layout))
      save_dlg.show()
    end;
  };
  {
    Button;
    text = "~~";
    contentDescription = "Buka File tersimpan";
    backgroundColor = "#000000";
    layout_width = "match_parent";
    layout_height = "wrap_content";
    onClick = function()
      local save_layout = {
        LinearLayout;
        orientation = "vertical";
        layout_width="match_parent";
        layout_height="match_parent";
        backgroundColor = "#000000";
        {
          ListView;
          id = "saved";
          backgroundColor = "#000000";
          layout_width = "match_parent";
          layout_height = "match_parent";
        };
      }
      saved_dlg = LuaDialog()
      saved_dlg.setTitle("File tersimpan")
      saved_dlg.setView(loadlayout(save_layout))
      saved_dlg.show()
      local chats_folder = "/sdcard/解说/SavedChats"
      local dir = io.popen("ls -t " .. chats_folder)
      local chats = {}
      for chat in dir:lines() do
        chat = chat:gsub(".txt", "")
        table.insert(chats, chat)
      end
      dir:close()
      saved.adapter = ArrayAdapter(service, android.R.layout.simple_list_item_1, chats)
      saved.onItemClick = function(parent, view, position, id)
        local chat = saved.getItemAtPosition(position)
        local filename = "/sdcard/解说/SavedChats/" .. chat .. ".txt"
        local file = io.open(filename, "r")
        for line in file:lines() do
          print(line)
        end
        file:close()
      end
    end;
  };
  {
    Button;
    text = "×";
    backgroundColor = "#000000";
    contentDescription = "Keluar";
    layout_width = "match_parent";
    layout_height = "wrap_content";
    onClick = function()
      dlg.dismiss()
    end;
  };
  {
    ScrollView;
    layout_width = "match_parent";
    layout_height = "match_parent";
    {
      LinearLayout;
      id = "messagesContainer";
      orientation = "vertical";
      layout_width = "match_parent";
      layout_height="wrap_content";
      padding = "10dp";
    };
  };
}

dlg = LuaDialog()
dlg.setTitle("ChatGPT Indo")
dlg.setView(loadlayout(layout))
dlg.show()
